# -*- coding: utf-8 -*-

import shortuuid
from CodeConvert import CodeConvert as cc
from django.conf import settings
from django.db.utils import IntegrityError
from django.shortcuts import redirect, render
from django_short_url.models import ShortURL
from furl import furl


def short_url_redirect(request, surl):
    """
    >> Chrome 请求
    ?nickname=姜戈

    >> Django 获取
    u'?nickname=%E5%A7%9C%E6%88%88'

    >> furl(lurl).add(furl(request.get_raw_uri()).query.params).url
    ?nickname=%C3%A5%C2%A7%C2%9C%C3%A6%C2%88%C2%88

    >> furl(lurl).add(furl(cc.Convert2Utf8(request.get_raw_uri())).query.params).url
    ?nickname=%E5%A7%9C%E6%88%88
    """
    try:
        lurl = ShortURL.objects.get(surl=surl).lurl
    except ShortURL.DoesNotExist:
        lurl = None

    # Short URL Not Exists
    if not lurl:
        redirect_url = ''

        if hasattr(settings, 'DJANGO_SHORT_URL_REDIRECT_URL'):
            redirect_url = settings.DJANGO_SHORT_URL_REDIRECT_URL

        if hasattr(settings, 'DJANGO_SHORT_URL_FUNC') and hasattr(settings.DJANGO_SHORT_URL_FUNC, '__call__'):
            redirect_url = settings.DJANGO_SHORT_URL_FUNC(request)

        if not redirect_url:
            return render(request, 'django_short_url/errmsg.html', {'title': 'Error', 'errmsg': 'Short URL not Exists'})

        return redirect(redirect_url)

    return redirect(furl(lurl).add(furl(cc.Convert2Utf8(request.get_raw_uri())).query.params).url)


def get_surl(lurl, length=None, domain=None, regex='s'):
    sobj, created = ShortURL.objects.get_or_create(lurl=lurl)
    if not sobj.surl:
        length = length or (getattr(settings, 'DJANGO_SHORT_URL_LENGTH') if hasattr(settings, 'DJANGO_SHORT_URL_LENGTH') else 0) or 22
        while True:
            sobj.surl = shortuuid.ShortUUID().random(length=length)
            try:
                sobj.save()
                break
            except IntegrityError:  # IntegrityError: (1062, "Duplicate entry 'xxx' for key 'surl'")
                continue
    return sobj.fsurl(domain=domain, regex=regex)



# -*- coding: utf-8 -*-

from django.contrib import admin
from django_admin import DeleteOnlyModelAdmin
from django_short_url.models import ShortURL


class ShortURLAdmin(DeleteOnlyModelAdmin, admin.ModelAdmin):
    list_display = ('surl', 'lurl', 'status', 'created_at', 'updated_at')


admin.site.register(ShortURL, ShortURLAdmin)



# -*- coding: utf-8 -*-

from django.conf.urls import include, url
from django_short_url import views as surl_views


app_name = 'django_short_url'


urlpatterns = [
    url(r'^(?P<surl>\w+)', surl_views.short_url_redirect, name='short_url_redirect'),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2017-10-25 13:23
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ShortURL',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('surl', models.CharField(blank=True, db_index=True, help_text='\u77ed\u94fe', max_length=32, null=True, unique=True, verbose_name='surl')),
                ('lurl', models.CharField(blank=True, db_index=True, help_text='\u957f\u94fe', max_length=255, null=True, unique=True, verbose_name='lurl')),
                ('status', models.BooleanField(db_index=True, default=True, help_text='\u662f\u5426\u663e\u793a', verbose_name='status')),
                ('created_at', models.DateTimeField(auto_now_add=True, help_text='\u521b\u5efa\u65f6\u95f4', verbose_name='created_at')),
                ('updated_at', models.DateTimeField(auto_now=True, help_text='\u66f4\u65b0\u65f6\u95f4', verbose_name='updated_at')),
            ],
            options={
                'verbose_name': 'shorturl',
                'verbose_name_plural': 'shorturl',
            },
        ),
    ]



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django_models_ext import BaseModelMixin


class ShortURL(BaseModelMixin):
    surl = models.CharField(_(u'surl'), max_length=32, blank=True, null=True, help_text=u'短链', db_index=True, unique=True)
    lurl = models.CharField(_(u'lurl'), max_length=255, blank=True, null=True, help_text=u'长链', db_index=True, unique=True)

    class Meta:
        verbose_name = _(u'shorturl')
        verbose_name_plural = _(u'shorturl')

    def __unicode__(self):
        return unicode(self.pk)

    @property
    def fdomain(self):
        if hasattr(settings, 'DJANGO_SHORT_URL_DOMAIN'):
            return settings.DJANGO_SHORT_URL_DOMAIN
        elif hasattr(settings, 'DOMAIN'):
            return settings.DOMAIN
        return ''

    def fsurl(self, domain=None, regex='s'):
        return u'{0}/{1}/{2}'.format(domain or self.fdomain, regex, self.surl)

    @property
    def data(self):
        return {
            'lurl': self.lurl,
            'surl': self.fsurl,
        }



# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2018-01-07 14:26
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_short_url', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='shorturl',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True, help_text='Create Time', verbose_name='created_at'),
        ),
        migrations.AlterField(
            model_name='shorturl',
            name='status',
            field=models.BooleanField(db_index=True, default=True, help_text='Status', verbose_name='status'),
        ),
        migrations.AlterField(
            model_name='shorturl',
            name='updated_at',
            field=models.DateTimeField(auto_now=True, help_text='Update Time', verbose_name='updated_at'),
        ),
    ]
